#!/bin/env python
import asyncio
import sys
from mitmproxy import options
from mitmproxy.tools.dump import DumpMaster
from clotho import Clotho


async def start_proxy(host, port):
    opts = options.Options(listen_host=host, listen_port=port)
    clotho = Clotho()
    m = DumpMaster(opts)

    m.addons.add(clotho)

    await m.run()
    return m


def cli():
    if len(sys.argv) != 3:
        print("Run with clotho listen_host listen_port, e.g. clotho localhost 8080")
        sys.exit(1)
    host = sys.argv[1]
    port = int(sys.argv[2])
    asyncio.run(start_proxy(host, port))


if __name__ == "__main__":
    cli()



import base64
import binascii
import re

import yaml

from mitmproxy import http


class AWSCredential:
    def __init__(self, access_key_id: str, date: str, region: str, service: str):
        self.access_key_id = access_key_id
        self.date = date
        self.region = region
        self.service = service
        try:
            self.account_id = self.get_account_id(self.access_key_id)
        except:
            self.account_id = "000000000000"

    def get_account_id(self) -> str:
        mask = 140737488355200
        id_type = self.access_key_id[4:]  # remove KeyID prefix
        id_bytes = base64.b32decode(id_type)  # base32 decode
        be_start = id_bytes[0:6]
        z = int.from_bytes(y, byteorder="big", signed=False)
        return (z & mask) >> 7


class Clotho:
    def __init__(self):
        with open("config.yaml.example") as f:
            self.config = yaml.load(f, Loader=yaml.CLoader)
        self.auth_re = re.compile(r"Credential=(A.*?)/(20.*?)/(.*?)/(.*?)/")

    def is_request_allowed(self, cred) -> bool:
        allowed_account = self.config.get(cred.account_id, self.config["accounts"].get("*", None))
        if not allowed_account:
            return False
        allowed_region = allowed_account["regions"].get(cred.region, allowed_account["regions"].get("*", None))
        if not allowed_region:
            return False
        if cred.service in allowed_region["services"] or "*" in allowed_region["services"]:
            return True
        return False

    async def request(self, flow):
        headers = dict(flow.request.headers)
        if "Authorization" not in headers:
            flow.response = http.Response.make(
                403,
                '{"Error":"Missing Authorization Token"}',
                {"content-type": "text/json"},
            )
            return
        try:
            parts = self.auth_re.findall(headers["Authorization"])[0]
            cred = AWSCredential(access_key_id=parts[0], date=parts[1], region=parts[2], service=parts[3])
        except Exception as e:
            flow.response = http.Response.make(
                400,
                '{"Error":"Malformed AWS SigV4 Request"}',
                {"content-type": "text/json"},
            )
            return
        else:
            try:
                if self.is_request_allowed(cred):
                    return
                else:
                    flow.response = http.Response.make(
                        403,
                        '{"Error":"Unauthorized"}',
                        {"content-type": "text/json"},
                    )
            except:
                flow.response = http.Response.make(400, '{"Error":"No account found"}', {"content-type": "text/json"})


addons = [Clotho()]



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()
setup(
    name="Clotho",
    version="0.1.0",
    author="CostasK",
    license="MIT",
    description="Proxy requests to AWS public endpoints and filter them using an allowlist",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ClothoProxy/PyClotho",
    py_modules=["clotho"],
    packages=find_packages(),
    install_requires=[requirements],
    python_requires=">=3.9",
    package_data={"": ["config.yaml.example"]},
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points="""
        [console_scripts]
        clotho=main:cli
    """,
    keywords="aws proxy cross-account security",
)
